// Algebra2.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include "SequenceElement.h"
#include "GroupDef.h"
#include "EdgeConditions.h"
#include "TypeDef.h"
#include "Utility.h"
#include "CommandLine.h"
#include "Exception.h"
#include "SequenceClasses.h"
#include "SequenceBlock.h"

val_less<GroupElement*>	g_valLess;
LengthMap_t				g_lengthMap;
OrderClassStats_t		g_orderClassStats;
MultiplicityClassStats_t	g_multiplicityClassStats;
OrderMultComboStats_t	g_OrderMultComboStats;
std::ofstream			output;
SequenceBlockLookup_t	g_BlockLookup;

#ifdef USE_WINDOWS_CODE
	DWORD	g_dwTimer = 0;
	DWORD	g_dwPart1Timer = 0;
	DWORD	g_dwPart2Timer = 0;
	DWORD	g_dwPart3Timer = 0;
	DWORD	g_dwPart4Timer = 0;
	DWORD	g_dwPart5Timer = 0;
	DWORD	g_dwPart6Timer = 0;
	DWORD	g_dwPart7Timer = 0;
	DWORD	g_dwPart8Timer = 0;
	DWORD	g_dwPartATimer = 0;
	DWORD	g_dwPartBTimer = 0;
	DWORD	g_dwExamined = 0;

	DWORD	g_dwMergeSeqTimer = 0;
	DWORD	g_dwOuterLoopTimer = 0;
	DWORD	g_dwInnerLoopTimer = 0;
#endif

double g_nChosenPruned = 0;
double g_nIgnored = 0;
long nNumFoundSequences = 0;

const long g_dwSecondsPerMinute = 60;

#ifdef USE_WINDOWS_CODE
	const DWORD g_dwSecondsPerHour = 60 * g_dwSecondsPerMinute;
	const DWORD g_dwSecondsPerDay = g_dwSecondsPerHour * 24;
	const DWORD g_dwSecondsPerWeek = g_dwSecondsPerDay * 7;
#endif

class inverse_exists
	: public std::binary_function<GroupElement*, Set_t, bool>
{
public:
	inverse_exists(const Group& rGroup)
		: m_rGroup(rGroup)
	{
	}

	bool operator()(const GroupElement* rElem, const Set_t& rSet) const
	{
		GroupElement* pInv = m_rGroup.Inverse(*rElem);

		Set_t::const_iterator iter = rSet.find(pInv);
		if (iter == rSet.end())
			return false;
		return true;
	}
private:
	const Group& m_rGroup;
};

struct edge_tracker
	: public std::binary_function<CodedSequence_t, CodedSequence_t, bool>
{
	edge_tracker(int d)
		: m_d(d), m_nCount(0)
	{
	}
	bool operator()(const CodedSequence_t& lhs, const CodedSequence_t& rhs) const
	{
		m_nCount++;
		return true;
	}

	int GetCount() const {return m_nCount;}
private:
	mutable int m_nCount;
	int m_d;
};

#ifdef USE_WINDOWS_CODE
std::string GetTimeString(DWORD dwMilliseconds)
{
	std::stringstream strTime;

	DWORD dwSeconds = dwMilliseconds / 1000;
	DWORD dwMinutes = 0;
	DWORD dwHours = 0;
	DWORD dwDays = 0;
	DWORD dwWeeks = 0;
	bool bRest = false;

	if (dwSeconds >= g_dwSecondsPerWeek)
	{
		dwWeeks = (dwSeconds - dwSeconds % g_dwSecondsPerWeek) / g_dwSecondsPerWeek;
		dwSeconds = dwSeconds % g_dwSecondsPerWeek;
	}

	if (dwSeconds >= g_dwSecondsPerDay)
	{
		dwDays = (dwSeconds - dwSeconds % g_dwSecondsPerDay) / g_dwSecondsPerDay;
		dwSeconds = dwSeconds % g_dwSecondsPerDay;
	}

	if (dwSeconds >= g_dwSecondsPerHour)
	{
		dwHours = (dwSeconds - dwSeconds % g_dwSecondsPerHour) / g_dwSecondsPerHour;
		dwSeconds = dwSeconds % g_dwSecondsPerHour;
	}

	if (dwSeconds >= g_dwSecondsPerMinute)
	{
		dwMinutes = (dwSeconds - dwSeconds % g_dwSecondsPerMinute) / g_dwSecondsPerMinute;
		dwSeconds = dwSeconds % g_dwSecondsPerMinute;
	}

	if (dwWeeks > 0)
	{
		strTime << dwWeeks << " week";
		if (dwWeeks != 1)
			strTime << "s";
		strTime << ", ";
		bRest = true;
	}

	if (dwDays > 0 || bRest)
	{
		strTime << dwDays << " day";
		if (dwDays != 1)
			strTime << "s";
		strTime << ", ";

		bRest = true;
	}

	if (dwHours > 0 || bRest)
	{
		strTime << dwHours << " hour";
		if (dwHours != 1)
			strTime << "s";
		strTime << ", ";
		bRest = true;
	}

	if (dwMinutes > 0 || bRest)
	{
		strTime << dwMinutes << " minute";
		if (dwMinutes != 1)
			strTime << "s";
		strTime << ", ";
		bRest = true;
	}

	strTime << dwSeconds << " second";
	if (dwSeconds != 1)
		strTime << "s";

	strTime << " elapsed...";

	return strTime.str();
}
#endif

void OutputSequence(FiniteAbelianGroup& g, const Sequence_t& rSequence, const CommandLineOptions& rOptions)
{

	++g_lengthMap[rSequence.size()];
	++nNumFoundSequences;

	std::string strOutput;

	OrderClass* pOrderClass = 
		(rOptions.DisplayOrders()) 
			? g.GetOrderClass(rSequence) 
			: NULL;

	std::auto_ptr<CodedSequence_t> apCoded;

	MultiplicityClass* pMultClass = NULL;
	if (rOptions.DisplayMultiplicity())
	{
		apCoded = std::auto_ptr<CodedSequence_t>(new CodedSequence_t);
		g.EncodeSequence(rSequence, *apCoded);

		pMultClass = g.GetMultiplicityClass(*apCoded);
	}

	std::stringstream streamOut;
	switch (rOptions.GetDispFormat())
	{
		case DispFormatStandard:
			streamOut << rSequence;
			break;
		case DispFormatMultiplicity:
		{
			if (apCoded.get() == NULL)
			{
				apCoded = std::auto_ptr<CodedSequence_t>(new CodedSequence_t);
				g.EncodeSequence(rSequence, *apCoded);
			}
			streamOut << *apCoded;
			break;
		}
	}

	if (pOrderClass)
	{
		g_orderClassStats[pOrderClass]++;
		streamOut << "\t" << *pOrderClass;
	}

	if (pMultClass)
	{
		g_multiplicityClassStats[pMultClass]++;
		streamOut << "\t" << *pMultClass;
	}

	if (pOrderClass && pMultClass)
		g_OrderMultComboStats[std::make_pair(pMultClass, pOrderClass)]++;

	strOutput = streamOut.str();


	if (rOptions.DisplayToScreen())
	{
		static Sequence_t lastSequenceOut(rSequence);

		int n = rOptions.GetProgressMinChangeElmt();
		std::pair<Sequence_t::const_iterator, Sequence_t::const_iterator> pair;

		pair = std::mismatch(rSequence.begin(), rSequence.end(), lastSequenceOut.begin());

		int dist = pair.first - rSequence.begin()+1;
		//dist = rSequence.size() - dist;

		if ((dist <= n) || (rSequence.size() > lastSequenceOut.size()) || (n == 0) || (pair.first == rSequence.end()))
		{
			lastSequenceOut = rSequence;
			std::cout << strOutput << std::endl;
		}
	}
	if (rOptions.UseSequenceFile())
		output << strOutput << std::endl;
}

void OutputInitialInfo(FiniteAbelianGroup& rGroup, const CommandLineOptions& rOptions)
{
	std::cout << "Group: " << rGroup.GetGroupString() << std::endl;

	std::cout << "Order of group: " << rGroup.order() << std::endl;
	std::cout << "Number of available sequence elements: " << rGroup.pool_size() << std::endl;

	if (rOptions.UseInputFile())
		std::cout << "Using input file " << rOptions.GetInputFile() << std::endl;
	else
		std::cout << "No input file specified.  Calculating..." << std::endl;

	if (rOptions.UseSequenceFile())
		std::cout << "Sending sequence output to " << rOptions.GetSequenceFile() << std::endl;

	if (rOptions.UseStatsFile())
		std::cout << "Sending statistics output to " << rOptions.GetStatsFile() << std::endl;

	std::cout << "Display Format: " << FormatDesc[rOptions.GetDispFormat()] << std::endl;

	if (rOptions.DisplayCrossNumbers() || rOptions.DisplayMultiplicity() || rOptions.DisplayOrders())
	{
		std::cout << "Display: ";
		bool bFirst = true;

		if (rOptions.DisplayCrossNumbers())
		{
			bFirst = false;
			std::cout << "Cross Numbers";
		}

		if (rOptions.DisplayMultiplicity())
		{
			if (!bFirst)
				std::cout << ", ";
			std::cout << "Multiplicities";
			bFirst = false;
		}

		if (rOptions.DisplayOrders())
		{
			if (!bFirst)
				std::cout << ", ";
			std::cout << "Orders";
		}

		std::cout << std::endl;
	}

}

void OutputStatistics(const CommandLineOptions& rOptions/*, DWORD dwTime*/)
{
	std::ofstream stats_file;
	stats_file.open(rOptions.GetStatsFile().c_str());

	std::cout << "----------Statistics----------" << std::endl;
	stats_file << "----------Statistics----------" << std::endl;
	//std::cout << g_FoundSequences.size() << " total sequences found" << std::endl;
	//stats_file << g_FoundSequences.size() << " total sequences found" << std::endl;
	std::cout << nNumFoundSequences << " total sequences found" << std::endl;
	stats_file << nNumFoundSequences << " total sequences found" << std::endl;

	LengthMap_t::const_iterator lengthMapIter = g_lengthMap.begin();
	LengthMap_t::const_iterator lengthMapIterEnd = g_lengthMap.end();
	for (; lengthMapIter != lengthMapIterEnd; ++lengthMapIter)
	{
		std::pair<int, int> curPair = *lengthMapIter;
		std::cout << "Length " << curPair.first << ": " << curPair.second << " total sequences" << std::endl;
		stats_file << "Length " << curPair.first << ": " << curPair.second << " total sequences" << std::endl;
	}

	std::cout << std::endl;
	stats_file << std::endl;

	if (rOptions.DisplayOrders())
	{
		std::cout << "----------Order Class Statistics----------" << std::endl;
		stats_file << "----------Order Class Statistics----------" << std::endl;
		OrderClassStats_t::const_iterator ocIter = g_orderClassStats.begin();
		OrderClassStats_t::const_iterator ocIterEnd = g_orderClassStats.end();
		for (; ocIter != ocIterEnd; ++ocIter)
		{
			const OrderClassStats_t::value_type& rPair = *ocIter;
			OrderClass* pClass = rPair.first;
			int nCount = rPair.second;
			std::stringstream str;
			str << *pClass << "\t" << nCount << " element";
			if (nCount != 1)
				str << "s";
			str << std::endl;
			std::cout << str.str();
			stats_file << str.str();
		}

		std::cout << g_orderClassStats.size() << " distinct order classes." << std::endl << std::endl;
		stats_file << g_orderClassStats.size() << " distinct order classes." << std::endl << std::endl;
	}

	if (rOptions.DisplayMultiplicity())
	{
		std::cout << "----------Multiplicity Class Statistics----------" << std::endl;
		stats_file << "----------Multiplicity Class Statistics----------" << std::endl;
		MultiplicityClassStats_t::const_iterator mcIter = g_multiplicityClassStats.begin();
		MultiplicityClassStats_t::const_iterator mcIterEnd = g_multiplicityClassStats.end();
		for (; mcIter != mcIterEnd; ++mcIter)
		{
			const MultiplicityClassStats_t::value_type& rPair = *mcIter;
			MultiplicityClass* pClass = rPair.first;
			int nCount = rPair.second;
			std::stringstream str;
			str << *pClass << "\t" << nCount << " element";
			if (nCount != 1)
				str << "s";
			str << std::endl;
			std::cout << str.str();
			stats_file << str.str();
		}

		std::cout << g_multiplicityClassStats.size() << " distinct multiplicity classes." << std::endl << std::endl;
		stats_file << g_multiplicityClassStats.size() << " distinct multiplicity classes." << std::endl << std::endl;
	}

	if (rOptions.DisplayOrders() && rOptions.DisplayMultiplicity())
	{
		std::cout << "----------Combined Order/Multiplicity Class Statistics----------" << std::endl;
		stats_file << "----------Combined Order/Multiplicity Class Statistics----------" << std::endl;

		OrderMultComboStats_t::const_iterator iter = g_OrderMultComboStats.begin();
		OrderMultComboStats_t::const_iterator iterEnd = g_OrderMultComboStats.end();
		for (; iter != iterEnd; ++iter)
		{
			const OrderMultComboStats_t::value_type& rPair = *iter;
			const OMPair_t& p = rPair.first;
			int n = rPair.second;

			std::stringstream streamOut;
			streamOut << *p.first << "\t" << *p.second << "\t- " << n << " element";
			if (n > 1)
				streamOut << "s";

			streamOut << std::endl;
			std::cout << streamOut.str();
			stats_file << streamOut.str();
		}

		std::cout << g_OrderMultComboStats.size() << " distinct combined order/multiplicity classes." << std::endl << std::endl;
		stats_file << g_OrderMultComboStats.size() << " distinct combined order/multiplicity classes." << std::endl << std::endl;
	}

	std::cout << std::endl;
	stats_file << std::endl;

	std::cout << g_nChosenPruned << " branches pruned" << std::endl;
	stats_file << g_nChosenPruned << " branches pruned" << std::endl;

	std::cout << g_nIgnored << " minimal zero sequences ignored" << std::endl;
	stats_file << g_nIgnored << " minimal zero sequences ignored" << std::endl;

	//std::string strTime = GetTimeString(dwTime);

	//std::cout << strTime << std::endl;
	//stats_file << strTime << std::endl;

	stats_file.close();
}

void GenerateSequences2(FiniteAbelianGroup& g, Sequence_t curSequence, const Set_t& rSums, Set_t forbidden, const CommandLineOptions& rOptions)
{
	Set_t chooseSet;
	
	//P = Set of elements which are possible for the _current_ element we'll be adding
	//F = Set of elements which are forever forbidden
	//NF = Set of elements which are forbidden to appear _after_ the node we're about to pick
	//		but might appear again later on _this_ level or after some node other than the one
	//		we're about to pick
	//S = Universal set of all elements which can possibly appear in any MZS we're interested in
	//	i.e. 0 will _never_ a member of U.
	//Su = Set of all sums and subsums of the current ZFS

	//P = S - F
	std::set_difference
	(
		g.pool_begin(), g.pool_end(), 
		forbidden.begin(), forbidden.end(), 
		std::inserter(chooseSet, chooseSet.end()),
		g_valLess
	);

	if (curSequence.size() >= rOptions.GetLengthLowerBound() - 1)
	{
		//Make a copy of the ZFS so we can sort it without damaging the order
		//(we will need to use it in its unmodified order to recurse)
		Sequence_t foundSequence(curSequence.begin(), curSequence.end());
		Sequence_t::const_iterator iter = foundSequence.begin();
		Sequence_t::const_iterator iterEnd = foundSequence.end();

		//pSum = Sum of the current ZFS
		GroupElement* pSum = g.GetZero();
		for (; iter != iterEnd; ++iter)
			pSum = g.CombineElements(*pSum, **iter);

		GroupElement* pInverse = g.Inverse(*pSum);
		GroupElement* pLast = (foundSequence.empty()) ? g.GetZero() : foundSequence.back();

		Set_t::const_iterator fIter = std::find(g.pool_begin(), g.pool_end(), pInverse);

		if (!g_valLess(pInverse, pLast) && (fIter != g.pool_end()))
		{
			//Now that we have the sum, tack it's inverse onto the end of the current 
			//ZFS to make an MZS

			foundSequence.push_back(pInverse);

			bool bSatifsiesReq = true;
			Set_t::const_iterator requiredIter = rOptions.required_begin();
			Set_t::const_iterator requiredIterEnd = rOptions.required_end();
			for (; bSatifsiesReq && requiredIter != requiredIterEnd; ++requiredIter)
			{
				GroupElement* pRequiredEl = *requiredIter;
				bSatifsiesReq = std::binary_search(foundSequence.begin(), foundSequence.end(), pRequiredEl, g_valLess);
			}

			if (bSatifsiesReq)
				OutputSequence(g, foundSequence, rOptions);
		}
	}
	else
		++g_nIgnored;

	if (curSequence.size() == rOptions.GetLengthUpperBound()-1)
		return;

	while (!chooseSet.empty())
	{
		//p = min{P}
		//Since chooseSet is automatically sorted, this is automatically the smallest element
		GroupElement* p = *chooseSet.begin();

		//P = P - {p}
		//This prevents p from being chosen again ONLY as a sibling of this choice
		chooseSet.erase(chooseSet.begin());

		//NF = F
		Set_t futureForbidden(forbidden);

		//F = F  U  {p}
		//This prevents p from being chosen again as a child of a sibling of this choice
		forbidden.insert(p);

		//NF = NF U {-p}
		//p.Inverse() is only forbidden for children of this choice
		futureForbidden.insert(g.Inverse(*p));
		
		//Create a new list of sums by the rule:
		//Su[n] = Su[n-1] U {p} U {p + Su[n-1]}
		
		//Takes care of U S[n-1] by initialization
		Set_t newSums(rSums);

		//Takes care of U {p}
		newSums.insert(p);
		
		//Takes care of U {p + S[n-1]}
		Set_t::const_iterator iter = rSums.begin();
		Set_t::const_iterator iterEnd = rSums.end();
		for (; iter != iterEnd; ++iter)
		{
			//This is the current sum we're looking at
			const GroupElement& rElem = **iter;

			//Add it to p
			GroupElement* pNewSum = g.CombineElements(rElem, *p);

			//Finally Su[n] = Su[n]  U  {newSum}
			newSums.insert(pNewSum);

			//NF = NF  U  {-pNewSum}
			//The inverse of this sum is only forbidden for children of this level
			futureForbidden.insert(g.Inverse(*pNewSum));
		}
		

		curSequence.push_back(p);
	
		int nElementOrder = g.order(*p);
		if (rOptions.IsOrderBounded(nElementOrder))
		{
			int nBound = rOptions.GetOrderBound(nElementOrder);
			int nCount = std::count(curSequence.begin(), curSequence.end(), p);
			if (nCount == nBound)
				futureForbidden.insert(p);
		}
	
		GenerateSequences2(g, curSequence, newSums, futureForbidden, rOptions);


		curSequence.pop_back();
	}
}

void GenerateSequences2(FiniteAbelianGroup& g, const CommandLineOptions& rOptions)
{
	Sequence_t sequence;
	Set_t sums;
	Set_t forbidden;

	GenerateSequences2(g, sequence, sums, forbidden, rOptions);
}

//#define STATS1
//#define STATS3
//#define STATS4
//#define STATS5
//Given length's a and b, merge all sequences of length a and b into sequences of length a+b
void MergeSequences(FiniteAbelianGroup& g, int a, int b, const CommandLineOptions& rOptions)
{
	SequenceBlockList_t* pLISTa = NULL;
	SequenceBlockList_t* pLISTb = NULL;

	{
		SequenceBlockLookup_t::iterator iter = g_BlockLookup.find(a); 
		if (iter == g_BlockLookup.end())
			g_BlockLookup[a] = pLISTa = new SequenceBlockList_t;
		else
			pLISTa = (*iter).second;

		iter = g_BlockLookup.find(b); 
		if (iter == g_BlockLookup.end())
			g_BlockLookup[b] = pLISTb = new SequenceBlockList_t;
		else
			pLISTb = (*iter).second;
	}


	SequenceBlockList_t& LISTa = *pLISTa;
	SequenceBlockList_t& LISTb = *pLISTb;

	SequenceBlockList_t* pNewList = NULL;
	{
		SequenceBlockLookup_t::iterator iter = g_BlockLookup.find(a+b); 
		if (iter == g_BlockLookup.end())
			g_BlockLookup[a+b] = pNewList = new SequenceBlockList_t;
		else
			pNewList = (*iter).second;
	}

	SequenceBlockList_t::const_iterator iter = LISTa.begin();
	SequenceBlockList_t::const_iterator iterEnd = LISTa.end();

	SequenceBlockList_t& newList = *pNewList;

	//For every element of LISTa
	for (; iter != iterEnd; ++iter)
	{
		//x = current element
		SequenceBlock& x = **iter;

		//Get the largest element in the current sequence
		//Sequence_t::const_iterator maxIter = std::max_element(x.sequence_begin(), x.sequence_end());
		GroupElement& rMaxElementSeq1 = *x.sequence_back();

		//For every element of LISTb
		Sequence_t lbSearchSeq;
		Set_t lbSearchSums;
		lbSearchSeq.push_back(&rMaxElementSeq1);
		lbSearchSums.insert(&rMaxElementSeq1);
		SequenceBlock block(g, lbSearchSeq, lbSearchSums);
		SequenceBlockList_t::const_iterator iter2 = LISTb.lower_bound(&block);
		SequenceBlockList_t::const_iterator iter2End = LISTb.end();
		for (; iter2 != iter2End; ++iter2)
		{
#ifdef USE_WINDOWS_CODE
			++g_dwExamined;
#endif

			SequenceBlock& y = **iter2;

			//Sequence_t::const_iterator minIter = std::min_element(y.sequence_begin(), y.sequence_end());
			const GroupElement& rMinElementSeq2 = *y.sequence_front();

			//Only look at the LISTb sequences whose smallest element is >= the current LISTa sequence's
			//largest element
			if (rMinElementSeq2 < rMaxElementSeq1)
			{
				Sequence_t s1(x.sequence_begin(), x.sequence_end());
				Sequence_t s2(y.sequence_begin(), y.sequence_end());
				std::cout << "s1 = " << s1 << ".  s2 = " << s2 << std::endl;
				continue;
			}

			//E = x.L2 | y.L1
			std::vector<bool>	E(g.order());

			std::transform(
				x.L2_begin(), x.L2_end(), 
				y.L1_begin(), 
				E.begin(), 
				std::logical_or<bool>()
			);


			//bEqual = (E == x.L2)
			bool bEqual = std::equal(E.begin(), E.end(), x.L2_begin());
			

			//Stick y's sequence onto the end of x's sequence
			int nNewSize = x.sequence_size() + y.sequence_size();
			Sequence_t mergedSequence(nNewSize);

			std::merge(
				x.sequence_begin(), x.sequence_end(), 
				y.sequence_begin(), y.sequence_end(), 
				mergedSequence.begin()
			);

			//if (E == x.L2) ... else
			if (bEqual)
			{
				//Get the list of MZSs of this length
				SequenceBlockList_t& rList = newList;

				//newL1 = x.L1 | y.L1
				std::vector<bool> newL1(g.order());


				std::transform
				(
					x.L1_begin(), x.L1_end(), 
					y.L1_begin(), newL1.begin(), 
					std::logical_or<bool>()
				);

				//For each a in x.L1
				//Convert to sequences first maybe
				std::vector<bool>::const_iterator xIter = x.L1_begin();
				std::vector<bool>::const_iterator xIterEnd = x.L1_end();
				std::vector<bool>::const_iterator yIter = y.L1_begin();
				std::vector<bool>::const_iterator yIterEnd = y.L1_end();
				int xDist = 0;
				int yDist = 0;

				for (; xIter != xIterEnd; ++xIter, ++xDist)
				{
					if (*xIter == false)
						continue;

					const GroupElement* pXElement = g.PositionToElement(xDist);

					//And for each b in y.L1
					for (yIter = y.L1_begin(), yDist=0; yIter != yIterEnd; ++yIter, ++yDist)
					{
						if (*yIter == false)
							continue;

						const GroupElement* pYElement = g.PositionToElement(yDist);

						//Calculate a + b
						GroupElement* pSum = g.CombineElements(*pXElement, *pYElement);

						int nNewPosition = g.ElementToPosition(*pSum);

						//Set (a+b)'s bit to true
						newL1[nNewPosition] = true;
					}
				}

				Set_t s;
				VectorBoolToSet(g, newL1, s);
				SequenceBlock* b = new SequenceBlock(g, mergedSequence, s);
				newList.insert(b);
			}
			else
			{
				//if (a + b == M)
				if (a + b == rOptions.GetDesiredBlockLength())
				{
					std::vector<bool> out(g.order());

					//out = x.L2 xor E
					std::transform(
						x.L2_begin(), x.L2_end(), 
						E.begin(), out.begin(), 
						logical_xor<bool>()
					);

					//Count the number of ON bits in out
					int n = std::count(out.begin(), out.end(), true);

					if (n == 1)
					{
						Sequence_t::const_iterator iterDest = mergedSequence.begin();
						Sequence_t::const_iterator iterDestEnd = mergedSequence.end();

						//Sum the whole sequence
						GroupElement* pSum = g.GetZero();
						for (; iterDest != iterDestEnd; ++iterDest)
							pSum = g.CombineElements(*pSum, **iterDest);
						
						if (pSum == g.GetZero())
							OutputSequence(g, mergedSequence, rOptions);
					}
				}
			}
		}
	}
}

void GenerateBlockSequences(FiniteAbelianGroup& g, const CommandLineOptions& rOptions)
{
	int nDesiredMZSLen = rOptions.GetDesiredBlockLength();
	int nDesiredZFSLen = nDesiredMZSLen - 1;
	int nBaseLength = 1;

	//The ZFS's of length 1 are basically just all elements from 1...n-1
	Set_t::const_iterator iter = g.pool_begin();
	Set_t::const_iterator iterEnd = g.pool_end();
	SequenceBlockList_t blockList;
	for (; iter != iterEnd; ++iter)
	{
		GroupElement* pElement = *iter;

		Sequence_t s;
		Set_t sums;

		s.push_back(pElement);
		sums.insert(pElement);

		SequenceBlock* b = new SequenceBlock(g, s, sums);

		blockList.insert(b);
	}

	g_BlockLookup[1] = &blockList;

	while (nBaseLength < nDesiredMZSLen)
	{
		SequenceBlockLookup_t::reverse_iterator iter2 = g_BlockLookup.rbegin();
		SequenceBlockLookup_t::reverse_iterator iter2End = g_BlockLookup.rend();

		while (true != false)
		{
			SequenceBlockLookup_t::value_type pair = *iter2;
			int n = (*iter2).first + nBaseLength;

			if (n > nDesiredMZSLen)
				++iter2;
			else
				break;
		}

		int nAppendLength = (*iter2).first;
		MergeSequences(g, nBaseLength, nAppendLength, rOptions);
		nBaseLength += nAppendLength;
	}
/*
	SequenceBlockList_t& rDesiredZFSList = g_BlockLookup[nDesiredZFSLen];
	SequenceBlockList_t::const_iterator zfsIter = rDesiredZFSList.begin();
	SequenceBlockList_t::const_iterator zfsIterEnd = rDesiredZFSList.end();
	for (; zfsIter != zfsIterEnd; ++zfsIter)
	{
		SequenceBlock& rBlock = **zfsIter;
		Sequence_t curSequence(rBlock.sequence_begin(), rBlock.sequence_end());
	}*/
}


int _tmain(int argc, char* argv[])
{
	std::vector<std::string>	argList;
	std::copy(&argv[1], &argv[argc], std::back_inserter(argList));


	try
	{
		CommandLineOptions options;

		if (argc == 1)
			throw CommandLineException("", options);

		options.Parse(argList);

		
		if (!options.GroupExists())
			throw CommandLineException("No group specified from command line!", options);
/*		{
			std::string strInput;
			std::vector<int>	values;
			//No input validity checking
			std::cout << "Enter the group size (numbers must be sorted in ascending order!): ";
			while (std::cin.peek() != 10)
			{
				int i;
				std::cin >> i;
				values.push_back(i);
			}

			options.AddGroup(values);
		}*/

		FiniteAbelianGroup& rGroup = options.GetGroup();

		//rGroup.Initialize(options);

		OutputInitialInfo(rGroup, options);

		//Generate the sequences
		if (options.UseSequenceFile())
		{
			std::string strFileName = options.GetSequenceFile();
			output.open(strFileName.c_str());
		}

#ifdef USE_WINDOWS_CODE
		DWORD dwTime = timeGetTime();
		if (options.UseBlockAlgorithm())
			GenerateBlockSequences(rGroup, options);
		else
			GenerateSequences2(rGroup, options);
		DWORD dwNewTime = timeGetTime();

		DWORD dwTimeDelta = dwNewTime - dwTime;
		if (options.UseSequenceFile())
			output.close();

		OutputStatistics(options, dwTimeDelta);
#else
		GenerateSequences2(rGroup, options);
		OutputStatistics(options);
#endif

/*		std::cout << "Part 1a: " << g_dwPart1Timer << " milliseconds" << std::endl;
		std::cout << "Part 2: " << g_dwPart2Timer << " milliseconds" << std::endl;
		std::cout << "Part 3: " << g_dwPart3Timer << " milliseconds" << std::endl;
		std::cout << "Part 4: " << g_dwPart4Timer << " milliseconds" << std::endl;
		std::cout << "Part 5: " << g_dwPart5Timer << " milliseconds" << std::endl;
		std::cout << "Part 6: " << g_dwPart6Timer << " milliseconds" << std::endl;
		std::cout << "Part A: " << g_dwPartATimer << " milliseconds" << std::endl;
		std::cout << "Part B: " << g_dwPartBTimer << " milliseconds" << std::endl;
		std::cout << "Total Function Time: " << g_dwMergeSeqTimer << " milliseconds" << std::endl;
		std::cout << "Outer Loop Time: " << g_dwOuterLoopTimer << " milliseconds" << std::endl;
		std::cout << "Inner Loop Time: " << g_dwInnerLoopTimer << " milliseconds" << std::endl;
		std::cout << "Total Sequences Examined: " << g_dwExamined << std::endl;

		std::cout << "Vector Less: " << g_dwTimer << " milliseconds" << std::endl;*/

	}
	catch (CommandLineException& e)
	{
		std::cerr << e.What() << std::endl;
	}
	catch(Exception& e)
	{
		std::cerr << "An error occured.  Reason: " << e.What() << std::endl;
	}
	return 0;
}

