#include "stdafx.h"
#include "SequenceElement.h"
#include "Utility.h"


DirectSumElement::~DirectSumElement()
{
}

DirectSumElement::DirectSumElement(const DirectSumElement& rhs)
	: GroupElement(rhs)
{
	m_pElementList.assign(rhs.begin(), rhs.end());
}


std::ostream& DirectSumElement::ToStream(std::ostream& rStream) const
{
	ElementList_t::const_iterator iter = m_pElementList.begin();
	ElementList_t::const_iterator iterEnd = m_pElementList.end();

	for (; iter != iterEnd; )
	{
		GroupElement* p = *iter;
		rStream << *p;
		if (++iter != iterEnd)
			rStream << ",";
	}

	return rStream;
}

bool DirectSumElement::operator==(const GroupElement& rhs) const
{
	const DirectSumElement& rhs1 = debug_dynamic_cast<const DirectSumElement&>(rhs);

	if (m_pElementList.size() != rhs1.m_pElementList.size())
		return false;

	val_compare<GroupElement*> comp;
	
	ElementList_t::const_iterator iter = m_pElementList.begin();
	ElementList_t::const_iterator iterEnd = m_pElementList.end();

	ElementList_t::const_iterator iter2 = rhs1.m_pElementList.begin();

	for (; iter != iterEnd; ++iter, ++iter2)
		if (!comp(*iter, *iter2))
			return false;

	return true;
}

bool DirectSumElement::operator<(const GroupElement& rhs) const
{
	VectorCompare_t compare;
	const DirectSumElement& rhs1 = debug_dynamic_cast<const DirectSumElement&>(rhs);

	//If both are special or neither are special, just compare them
	if (GetSpecialOrdering() == rhs.GetSpecialOrdering())
		return compare(m_pElementList, rhs1.m_pElementList);

	//Otherwise this one is less if it's special, else greater
	return GetSpecialOrdering();
}

void DirectSumElement::SetElement(int nIndex, GroupElement* pElement)
{
	m_pElementList[nIndex] = pElement;
}




bool IntModElement::operator == (const GroupElement& rhs) const
{
	const IntModElement& rhs1 = debug_dynamic_cast<const IntModElement&>(rhs);
	return m_nVal == rhs1.m_nVal;
}


bool IntModElement::operator<(const GroupElement& rhs) const
{
	const IntModElement& rhs1 = debug_dynamic_cast<const IntModElement&>(rhs);

	//If both are special or neither are special, just compare them
	if (GetSpecialOrdering() == rhs.GetSpecialOrdering())
		return m_nVal < rhs1.m_nVal;

	//Otherwise this one is less if it's special, else greater
	return GetSpecialOrdering();
}

std::ostream& IntModElement::ToStream(std::ostream& rStream) const
{
	rStream << m_nVal;
	return rStream;
}

IntModElement IntModElement::operator+(const IntModElement& rhs) const
{
	return IntModElement(m_nMod, m_nVal + rhs.m_nVal);
}



