#ifndef __SEQUENCELEMENT_H__
#define __SEQUENCELEMENT_H__

#include "Arithmetic.h"
#include "Algorithm.h"

class GroupElement
{
public:
	GroupElement()
		: m_bSpecialOrdering(false)
	{
	}

	GroupElement(const GroupElement& rhs)
		: m_bSpecialOrdering(rhs.m_bSpecialOrdering)
	{
	}

	virtual ~GroupElement() {}	

	virtual std::ostream& ToStream(std::ostream& rStream) const = 0;

	virtual bool operator < (const GroupElement& rhs) const = 0;
	virtual bool operator == (const GroupElement& rhs) const = 0;

	bool operator != (const GroupElement& rhs) const
	{
		return !(*this == rhs);
	}

	void	SetSpecialOrdering()	{m_bSpecialOrdering = true;}
	bool	GetSpecialOrdering() const	{return m_bSpecialOrdering;}

protected:
	bool	m_bSpecialOrdering;
};

class IntModElement
	: public GroupElement
{
public:
	IntModElement(int nMod, int val=0)
		: m_nVal(val % nMod), m_nMod(nMod)
	{

	}

	IntModElement(const IntModElement& rhs)
		: GroupElement(rhs)
		, m_nVal(rhs.m_nVal), m_nMod(rhs.m_nMod)
	{
	}

	int GetVal() const {return m_nVal;}

	virtual bool operator<(const GroupElement& rhs) const;
	virtual bool operator==(const GroupElement& rhs) const;
	IntModElement operator+(const IntModElement& rhs) const;
	virtual std::ostream& ToStream(std::ostream& rStream) const;

private:
	int		m_nVal;
	int		m_nMod;
};

class DirectSumElement
	: public GroupElement
{
	typedef vector_less<GroupElement*, val_less<GroupElement*> >	VectorCompare_t;
	typedef std::vector<GroupElement*>	ElementList_t;

public:
	DirectSumElement(int nLength)
		: m_pElementList(nLength)
	{
	}
	~DirectSumElement();

	DirectSumElement(const DirectSumElement& rhs);

	typedef ElementList_t::const_iterator	const_iterator;
	typedef ElementList_t::iterator			iterator;
	typedef ElementList_t::size_type		size_type;

	virtual std::ostream& ToStream(std::ostream& rStream) const;

	virtual bool operator==(const GroupElement& rhs) const;
	virtual bool operator<(const GroupElement& rhs) const;

	const_iterator	begin() const	{return m_pElementList.begin();}
	iterator		begin()			{return m_pElementList.begin();}

	const_iterator	end()	const	{return m_pElementList.end();}
	iterator		end()			{return m_pElementList.end();}

	size_type		length() const	{return m_pElementList.size();}

	void SetElement(int nIndex, GroupElement* pElement);

private:
	ElementList_t	m_pElementList;
	bool 			m_bSpecialOrder;
};

inline std::ostream& operator << (std::ostream& stream, const GroupElement& elem)
{
	return elem.ToStream(stream);
}

#endif