#include "stdafx.h"
#include "Utility.h"
#include "Typedef.h"

#include "GroupDef.h"
#include "SequenceElement.h"

std::ostream& operator << (std::ostream& stream, const Sequence_t& rSequence)
{
	Sequence_t::const_iterator iter = rSequence.begin();
	Sequence_t::const_iterator iterEnd = rSequence.end();
	
	for (; iter != iterEnd; )
	{
		stream << **iter;
		if (++iter != iterEnd)
			stream << " + ";
	}
	
	return stream;
}

std::ostream& operator << (std::ostream& stream, const Set_t& rSet)
{
	Set_t::const_iterator iter = rSet.begin();
	Set_t::const_iterator iterEnd = rSet.end();
	
	stream << "{";
	for (; iter != iterEnd; )
	{
		stream << **iter;

		if (++iter != iterEnd)
			stream << ",";
	}
	stream << "}";
	
	return stream;
}

std::ostream& operator << (std::ostream& stream, const CodedSequence_t& rSequence)
{
	CodedSequence_t::const_iterator iter = rSequence.begin();
	CodedSequence_t::const_iterator iterEnd = rSequence.end();
	
	for (; iter != iterEnd; )
	{
		stream << *iter->first;
		if (iter->second > 1)
			stream << "^" << iter->second;

		if (++iter != iterEnd)
			stream << " + ";
	}
	
	return stream;
}

CodedSequence_t sequence_encoder::operator()(const Sequence_t& s) const
{
	CodedSequence_t c;
	m_g.EncodeSequence(s, c);
	return c;
}

bool str_starts_with::operator()(const std::string& strLHS, const std::string& strRHS) const
{
	if (strRHS.length() > strLHS.length())
		return false;

	int n = strLHS.compare(0, strRHS.length(), strRHS);
	return (n == 0);
}

bool StringTokenizer::Done() const
{
	return (m_Pos == std::string::npos);
}

std::string	StringTokenizer::Next()
{
	std::string::size_type pos2 = m_str.find(m_Delim, m_Pos);
	
	if (pos2 == std::string::npos)
	{
		std::string str(m_str, m_Pos, std::string::npos);
		m_Pos = std::string::npos;
		return str;
	}
	std::string str(m_str, m_Pos, pos2 - m_Pos + 1);

	m_Pos = pos2 + 1;

	return str;
}

void StringTokenizer::Begin(const std::string& str, char delim)
{
	m_str = str;
	m_Delim = delim;
	m_Pos = 0;
}



void SetToVectorBool(const FiniteAbelianGroup& g, const Set_t& rSet, std::vector<bool>& rVector)
{
	rVector.clear();
	rVector.resize(g.order());

	Set_t::const_iterator iter = rSet.begin();
	Set_t::const_iterator iterEnd = rSet.end();

	for (; iter != iterEnd; ++iter)
	{
		const GroupElement* el = *iter;
		int n = g.ElementToPosition(*el);
		rVector[n] = true;
	}
}

void VectorBoolToSet(const FiniteAbelianGroup& g, const std::vector<bool>& rVector, Set_t& rSet)
{
	rSet.clear();

	VectorBoolToSet(g, rVector.begin(), rVector.end(), rSet);
}

void VectorBoolToSet(const FiniteAbelianGroup& g, std::vector<bool>::const_iterator iter, std::vector<bool>::const_iterator iterEnd, Set_t& rSet)
{
	std::vector<bool>::const_iterator iterBegin = iter;
	for (; iter != iterEnd; ++iter)
	{
		bool b = *iter;
		if (!b)
			continue;

		GroupElement* pElement = g.PositionToElement(iter - iterBegin);
		rSet.insert(pElement);
	}
}