#ifndef __ALGORITHM_H__
#define __ALGORITHM_H__

template<class T, class U, class Op1, class Op2>
struct pair_less
	: public std::binary_function<std::pair<T, U>, std::pair<T, U>, bool>
{
	bool operator()(const std::pair<T, U>& X, const std::pair<T, U>& Y)
	{
		if (comp1(X.first, Y.first))
			return true;

		if (!comp1(Y.first, X.first) && comp2(X.second, Y.second))
			return true;

		return false;
	}

private:
	Op1 comp1;
	Op2 comp2;
};


template<class T>
struct val_equal_to
	: public std::binary_function<T, T, bool>
{
	bool operator()(T lhs, T rhs) const
	{
		return *lhs == *rhs;
	}
};

template<class T>
struct logical_xor 
	: public std::binary_function<T, T, bool>
{
	bool operator()(const T& X, const T& Y) const
	{
		return (X ^ Y);
	}
};


template<class T, class Comp = val_equal_to<T> >
struct val_compare
	: public std::binary_function<T, T, bool>
{
	bool operator()(T lhs, T rhs) const
	{
		return comp(lhs, rhs);
	}
private:
	Comp comp;
};

template<class T>
struct val_less
	: public std::binary_function<T, T, bool>
{
	bool operator()(T lhs, T rhs) const
	{
		return *lhs < *rhs;
	}
};

template<class T, class U>
struct vector_less
	: public std::binary_function<std::vector<T>, std::vector<T>, bool>
{
	bool operator()(const std::vector<T>& rLHS, const std::vector<T>& rRHS) const
	{
		int nSize = mymin(rLHS.size(), rRHS.size());

		first_argument_type::const_iterator iter1 = rLHS.begin();
		second_argument_type::const_iterator iter2 = rRHS.begin();

		for (int i=0; i < nSize; ++i, ++iter1, ++iter2)
		{
			if (compare(*iter1, *iter2))
				return true;
			else if (compare(*iter2, *iter1))
				return false;
		}

		return (rRHS.size() > rLHS.size()) ? true : false;
	}
private:
	U compare;
};

template<class K, class V, class U>
struct map_less
	: public std::binary_function<std::map<K, V>, std::map<K, V>, bool>
{
	bool operator()(const std::map<K, V>& rLHS, const std::map<K, V>& rRHS) const
	{
		if (rLHS.size() < rRHS.size())
			return true;
		if (rLHS.size() > rRHS.size())
			return false;
		first_argument_type::const_iterator iter1 = rLHS.begin();
		first_argument_type::const_iterator iter1End = rLHS.end();

		second_argument_type::const_iterator iter2 = rRHS.begin();
		second_argument_type::const_iterator iter2End = rRHS.end();

		for (; iter1 != iter1End; ++iter1, ++iter2)
		{
			if (compare(*iter1, *iter2))
				return true;
			else if (compare(*iter2, *iter1))
				return false;
		}

		return false;
	}

private:
	U	compare;
};

#endif